// LICENSE : MIT
"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var CommitVisitor = function () {
    function CommitVisitor() {
        _classCallCheck(this, CommitVisitor);
    }

    _createClass(CommitVisitor, [{
        key: "visit",


        /**
         * @param commit
         * @param {Function} enter
         * @returns {Promise}
         */
        value: function visit(commit, enter) {
            var Types = CommitVisitor.Types;
            var BREAK = CommitVisitor.BREAK;
            var isCanceled = false;
            return commit.getDiff().then(function (diffList) {
                if (isCanceled) {
                    return;
                }
                return Promise.all(diffList.map(function (diff) {
                    if (isCanceled) {
                        return;
                    }
                    isCanceled = enter(Types.diff, diff) === BREAK;
                    return diff.patches().then(function (patches) {
                        if (isCanceled) {
                            return;
                        }
                        return Promise.all(patches.map(function (patch) {
                            if (isCanceled) {
                                return;
                            }
                            isCanceled = enter(Types.patch, patch) === BREAK;
                            return patch.hunks().then(function (hunks) {
                                if (isCanceled) {
                                    return;
                                }
                                return Promise.all(hunks.map(function (hunk) {
                                    if (isCanceled) {
                                        return;
                                    }
                                    isCanceled = enter(Types.hunk, hunk) === BREAK;
                                    return hunk.lines().then(function (lines) {
                                        if (isCanceled) {
                                            return;
                                        }
                                        isCanceled = enter(Types.lines, lines) === BREAK;
                                        lines.forEach(function (line) {
                                            if (isCanceled) {
                                                return;
                                            }
                                            isCanceled = enter(Types.line, line) === BREAK;
                                        });
                                    });
                                }));
                            });
                        }));
                    });
                }));
            });
        }
    }], [{
        key: "Types",
        get: function get() {
            return {
                "diff": "diff",
                "patch": "patch",
                "hunk": "hunk",
                "lines": "lines",
                "line": "line"
            };
        }

        /**
         * @return {string}
         */

    }, {
        key: "BREAK",
        get: function get() {
            return "BREAK";
        }
    }]);

    return CommitVisitor;
}();

exports.default = CommitVisitor;
//# sourceMappingURL=CommitVisitor.js.map