// LICENSE : MIT
"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _CommitVisitor = require("./CommitVisitor");

var _CommitVisitor2 = _interopRequireDefault(_CommitVisitor);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Git = require("nodegit");
var path = require("path");
var minimatch = require("minimatch");
var assert = require("assert");

/**
 * @typedef {Object} ModifiedDiff
 * @property {string} commitMessage
 * @property {string} deleted
 * @property {string} added
 * @property {string} filePath
 */

var GitHistory = function () {
    function GitHistory(repositoryPath) {
        _classCallCheck(this, GitHistory);

        assert(repositoryPath, "`repositoryPath` should be set");
        /**
         * @type {string}
         */
        this.repostiroyPath = path.resolve(process.cwd(), repositoryPath);
    }

    /**
     * @param {string} filePattern
     * @param {string} branch
     * @returns {Promise.<ModifiedDiff[]>}
     */


    _createClass(GitHistory, [{
        key: "getModifiedDiffs",
        value: function getModifiedDiffs(_ref) {
            var _this = this;

            var filePattern = _ref.filePattern;
            var _ref$branch = _ref.branch;
            var branch = _ref$branch === undefined ? "master" : _ref$branch;

            assert(filePattern, "`filePattern` should be set. e.g.) `*.md`");
            var visitor = new _CommitVisitor2.default();
            return new Promise(function (resolve, reject) {
                Git.Repository.open(_this.repostiroyPath).then(function (repo) {
                    return repo.getBranchCommit(branch);
                })
                // Display information about commits on master.
                .then(function (firstCommitOnMaster) {
                    // Create a new history event emitter.
                    var history = firstCommitOnMaster.history();
                    var promises = [];
                    var diffList = [];
                    // Listen for commit events from the history.
                    var useFilePath = undefined;
                    history.on("commit", function (commit) {
                        var commitMessage = commit.message();
                        var promise = visitor.visit(commit, function (type, payload) {
                            if (type === _CommitVisitor2.default.Types.patch) {
                                var patch = payload;
                                if (!patch.isModified()) {
                                    return;
                                }
                                var filePath = patch.newFile().path();
                                if (!minimatch(filePath, filePattern)) {
                                    return _CommitVisitor2.default.BREAK;
                                }
                                useFilePath = filePath;
                            }
                            if (type === _CommitVisitor2.default.Types.lines) {
                                (function () {
                                    var lines = payload;
                                    var diff = {
                                        commitMessage: commitMessage,
                                        filePath: useFilePath
                                    };
                                    lines.forEach(function (line) {
                                        if (diff["deleted"] && diff["added"]) {
                                            diffList.push(diff);
                                            diff = {
                                                commitMessage: commitMessage,
                                                filePath: useFilePath
                                            };
                                        }
                                        var origin = line.origin();
                                        if (String.fromCharCode(origin) === "-") {
                                            diff["deleted"] = line.content();
                                        } else if (String.fromCharCode(origin) === "+") {
                                            diff["added"] = line.content();
                                        }
                                    });
                                })();
                            }
                        });
                        promises.push(promise);
                    });

                    history.on('end', function () {
                        Promise.all(promises).then(function () {
                            resolve(diffList);
                        });
                    });
                    history.on("error", function (error) {
                        reject(error);
                    });
                    // Start emitting events.
                    history.start();
                }).catch(reject);
            });
        }
    }]);

    return GitHistory;
}();

exports.default = GitHistory;
//# sourceMappingURL=GitHistory.js.map