// LICENSE : MIT
"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ObjectAssign = require("object.assign");
var tokenize = require("kuromojin").tokenize;
var stripMarkdown = require('strip-markdown');
var remark = require("remark")().use(stripMarkdown);
/**
 *create matcher tokens
 */

var MatcherTokenizer = function () {
    function MatcherTokenizer() {
        _classCallCheck(this, MatcherTokenizer);
    }

    _createClass(MatcherTokenizer, null, [{
        key: "toTokenize",

        /**
         * @typedef {Object} ModifiedTokenDiff
         * @property {Object[]} oldTokens
         * @property {string} oldText
         * @property {Object[]} newTokens
         * @property {string} newText
         */
        /**
         * @typedef {Object} ModifiedTokenMatchDiff
         * @property {Object[]} oldTokens
         * @property {string} oldText
         * @property {Object[]} newTokens
         * @property {string} newText
         * @property {number} start
         * @property {number} end
         */
        /**
         * @param {ModifiedDiff[]} modifiedDiffs
         * @returns {Promise.<ModifiedTokenDiff[]>}
         */
        value: function toTokenize(modifiedDiffs) {
            var removeWordPosition = function removeWordPosition(token) {
                delete token.word_position;
                return token;
            };
            return new Promise(function (resolve, reject) {
                var promises = modifiedDiffs.map(function (modifiedDiff) {
                    var added = modifiedDiff.added;
                    var deleted = modifiedDiff.deleted;

                    var oldText = deleted;
                    try {
                        oldText = remark.process(deleted).contents.trim();
                    } catch (e) {}
                    var newText = added;
                    try {
                        newText = remark.process(added).contents.trim();
                    } catch (e) {}
                    var result = {
                        oldText: oldText,
                        newText: newText
                    };
                    var addedTokenPromise = tokenize(newText);
                    var deletedTokenPromise = tokenize(oldText);
                    return Promise.all([deletedTokenPromise, addedTokenPromise]).then(function (_ref) {
                        var _ref2 = _slicedToArray(_ref, 2);

                        var deletedTokens = _ref2[0];
                        var addedTokens = _ref2[1];

                        result.newTokens = addedTokens.map(removeWordPosition);
                        result.oldTokens = deletedTokens.map(removeWordPosition);
                        return result;
                    });
                });
                Promise.all(promises).then(resolve, reject);
            });
        }

        /**
         * 異なるtoken + 前後1 のtokenを集めた返す
         * @param tokenDiff
         * @returns {ModifiedTokenMatchDiff[]}
         */

    }, {
        key: "createTokenDiffs",
        value: function createTokenDiffs(tokenDiff) {
            var results = [];
            var result = { oldTokens: [], newTokens: [], start: undefined, end: undefined };
            var isDifferent = false;
            var oldTokens = tokenDiff.oldTokens;
            var newTokens = tokenDiff.newTokens;

            oldTokens.forEach(function (oldToken, index) {
                var newToken = newTokens[index];
                if (!newToken) {
                    return;
                }
                if (newToken.pos === oldToken.pos && newToken.surface_form === oldToken.surface_form) {
                    // 3
                    if (isDifferent) {
                        result.oldTokens.push(oldToken);
                        result.newTokens.push(newToken);
                        results.push(result);
                        result = { oldTokens: [], newTokens: [], start: undefined, end: undefined };
                    }
                    isDifferent = false;
                    return;
                }
                isDifferent = true;
                // 1
                var prevOldToken = oldTokens[index - 1];
                var prevNewToken = newTokens[index - 1];
                if (prevOldToken) {
                    if (result.oldTokens.indexOf(prevOldToken) === -1) {
                        result.oldTokens.push(prevOldToken);
                        if (prevNewToken) {
                            result.newTokens.push(prevNewToken);
                        }
                    }
                }
                // 2
                // record start
                if (result.start === undefined) {
                    result.start = result.oldTokens.length;
                }
                result.oldTokens.push(oldToken);
                result.newTokens.push(newToken);
                // update end
                result.end = result.oldTokens.length;
            });
            return results.map(function (result) {
                var oldText = result.oldTokens.map(function (token) {
                    return token.surface_form;
                }).join("");
                var newText = result.newTokens.map(function (token) {
                    return token.surface_form;
                }).join("");
                return ObjectAssign({}, result, {
                    oldText: oldText,
                    newText: newText
                });
            });
        }
    }]);

    return MatcherTokenizer;
}();

exports.default = MatcherTokenizer;
//# sourceMappingURL=MatcherTokenizer.js.map