"use strict";

var _GitHistory = require("./GitHistory");

var _GitHistory2 = _interopRequireDefault(_GitHistory);

var _MatcherTokenizer = require("./MatcherTokenizer");

var _MatcherTokenizer2 = _interopRequireDefault(_MatcherTokenizer);

var _DiffFilter = require("./DiffFilter");

var _DiffFilter2 = _interopRequireDefault(_DiffFilter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = require("debug")("git-log-to-errata");

/**
 * @typedef {Object} ErrataTokenDiff
 * @property {number} id
 * @property {Object[]} oldTokens
 * @property {string} oldText
 * @property {Object[]} newTokens
 * @property {string} newText
 * @property {number} start
 * @property {number} end
 */
/**
 *
 * @param {string} repository
 * @param {string} [branch
 * @param {string} [pattern]
 * @param {function} [preFilter]
 * @param {function} [postFilter]
 * @returns {Promise.<ErrataTokenDiff>}
 */
module.exports = function (_ref) {
    var repository = _ref.repository;
    var _ref$branch = _ref.branch;
    var branch = _ref$branch === undefined ? "master" : _ref$branch;
    var _ref$pattern = _ref.pattern;
    var pattern = _ref$pattern === undefined ? "**/*.md" : _ref$pattern;
    var preFilter = _ref.preFilter;
    var postFilter = _ref.postFilter;

    var history = new _GitHistory2.default(repository);
    return history.getModifiedDiffs({
        filePattern: pattern,
        branch: branch
    }).then(function (modifiedDiffs) {
        debug("pre diffs", modifiedDiffs.length);
        var filteredDiffs = _DiffFilter2.default.preFilter(modifiedDiffs, preFilter);
        return _MatcherTokenizer2.default.toTokenize(filteredDiffs).then(function (modifiedTokenDiffs) {
            var resultModifiedTokenDiffs = [];
            modifiedTokenDiffs.forEach(function (tokenDiff) {
                var array = _MatcherTokenizer2.default.createTokenDiffs(tokenDiff);
                resultModifiedTokenDiffs = resultModifiedTokenDiffs.concat(array);
            });
            var errataDiffs = resultModifiedTokenDiffs.map(function (diff, index) {
                diff.id = index + 1;
                return diff;
            });
            var postFilteredTokenDiffs = _DiffFilter2.default.postFilter(errataDiffs, postFilter);
            debug("post diffs", postFilteredTokenDiffs.length);
            return postFilteredTokenDiffs;
        });
    });
};
//# sourceMappingURL=git-log-to-errata.js.map